@@ -1,6 +1,5 @@
 /* Copyright (c) 2001-2004, Roger Dingledine.
- * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2020, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -44,9 +43,7 @@
 #include "lib/compress/compress.h"
 #include "app/config/config.h"
 #include "core/or/connection_edge.h"
-#include "feature/rend/rendcommon.h"
-#include "feature/rend/rendcache.h"
-#include "feature/rend/rendparse.h"
+#include "core/or/extendinfo.h"
 #include "test/test.h"
 #include "core/mainloop/mainloop.h"
 #include "lib/memarea/memarea.h"
@@ -57,12 +54,10 @@
 #include "core/or/policies.h"
 #include "app/config/statefile.h"
 #include "lib/crypt_ops/crypto_curve25519.h"
+#include "feature/nodelist/networkstatus.h"
 
 #include "core/or/extend_info_st.h"
 #include "core/or/or_circuit_st.h"
-#include "feature/rend/rend_encoded_v2_service_descriptor_st.h"
-#include "feature/rend/rend_intro_point_st.h"
-#include "feature/rend/rend_service_descriptor_st.h"
 #include "feature/relay/onion_queue.h"
 
 /** Run unit tests for the onion handshake code. */
@@ -354,6 +349,106 @@ test_onion_queues(void *arg)
   tor_free(onionskin);
 }
 
+static int32_t cbtnummodes = 10;
+
+static int32_t
+mock_xm_networkstatus_get_param(
+    const networkstatus_t *ns, const char *param_name, int32_t default_val,
+    int32_t min_val, int32_t max_val)
+{
+  (void)ns;
+  (void)default_val;
+  (void)min_val;
+  (void)max_val;
+  // only support cbtnummodes right now
+  tor_assert(strcmp(param_name, "cbtnummodes")==0);
+  return cbtnummodes;
+}
+
+static void
+test_circuit_timeout_xm_alpha(void *arg)
+{
+  circuit_build_times_t cbt;
+  build_time_t Xm;
+  int alpha_ret;
+  circuit_build_times_init(&cbt);
+  (void)arg;
+
+  /* Plan:
+   * 1. Create array of build times with 10 modes.
+   * 2. Make sure Xm calc is sane for 1,3,5,10,15,20 modes.
+   * 3. Make sure alpha calc is sane for 1,3,5,10,15,20 modes.
+   */
+
+  /* 110 build times, 9 modes, 8 mode ties, 10 abandoned */
+  build_time_t circuit_build_times[] = {
+    100, 20, 1000, 500, 200, 5000, 30, 600, 200, 300, CBT_BUILD_ABANDONED,
+    101, 21, 1001, 501, 201, 5001, 31, 601, 201, 301, CBT_BUILD_ABANDONED,
+    102, 22, 1002, 502, 202, 5002, 32, 602, 202, 302, CBT_BUILD_ABANDONED,
+    103, 23, 1003, 503, 203, 5003, 33, 603, 203, 303, CBT_BUILD_ABANDONED,
+    104, 24, 1004, 504, 204, 5004, 34, 604, 204, 304, CBT_BUILD_ABANDONED,
+    105, 25, 1005, 505, 205, 5005, 35, 605, 205, 305, CBT_BUILD_ABANDONED,
+    106, 26, 1006, 506, 206, 5006, 36, 606, 206, 306, CBT_BUILD_ABANDONED,
+    107, 27, 1007, 507, 207, 5007, 37, 607, 207, 307, CBT_BUILD_ABANDONED,
+    108, 28, 1008, 508, 208, 5008, 38, 608, 208, 308, CBT_BUILD_ABANDONED,
+    109, 29, 1009, 509, 209, 5009, 39, 609, 209, 309, CBT_BUILD_ABANDONED
+  };
+
+  memcpy(cbt.circuit_build_times, circuit_build_times,
+         sizeof(circuit_build_times));
+  cbt.total_build_times = 110;
+
+  MOCK(networkstatus_get_param, mock_xm_networkstatus_get_param);
+
+#define CBT_ALPHA_PRECISION 0.00001
+  cbtnummodes = 1;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 205);
+  tt_assert(fabs(cbt.alpha - 1.394401) < CBT_ALPHA_PRECISION);
+
+  cbtnummodes = 3;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 117);
+  tt_assert(fabs(cbt.alpha - 0.902313) < CBT_ALPHA_PRECISION);
+
+  cbtnummodes = 5;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 146);
+  tt_assert(fabs(cbt.alpha - 1.049032) < CBT_ALPHA_PRECISION);
+
+  cbtnummodes = 10;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 800);
+  tt_assert(fabs(cbt.alpha - 4.851754) < CBT_ALPHA_PRECISION);
+
+  cbtnummodes = 15;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 800);
+  tt_assert(fabs(cbt.alpha - 4.851754) < CBT_ALPHA_PRECISION);
+
+  cbtnummodes = 20;
+  Xm = circuit_build_times_get_xm(&cbt);
+  alpha_ret = circuit_build_times_update_alpha(&cbt);
+  tt_int_op(alpha_ret, OP_EQ, 1);
+  tt_int_op(Xm, OP_EQ, 800);
+  tt_assert(fabs(cbt.alpha - 4.851754) < CBT_ALPHA_PRECISION);
+
+ done:
+#undef CBT_ALPHA_PRECISION
+  UNMOCK(networkstatus_get_param);
+  circuit_build_times_free_timeouts(&cbt);
+}
+
 static void
 test_circuit_timeout(void *arg)
 {
@@ -372,7 +467,6 @@ test_circuit_timeout(void *arg)
   double timeout1, timeout2;
   or_state_t *state=NULL;
   int i, runs;
-  double close_ms;
   (void)arg;
 
   initialize_periodic_events();
@@ -393,18 +487,11 @@ test_circuit_timeout(void *arg)
   circuit_build_times_initial_alpha(&initial,
                                     CBT_DEFAULT_QUANTILE_CUTOFF/100.0,
                                     timeout0);
-  close_ms = MAX(circuit_build_times_calculate_timeout(&initial,
-                             CBT_DEFAULT_CLOSE_QUANTILE/100.0),
-                 CBT_DEFAULT_TIMEOUT_INITIAL_VALUE);
   do {
     for (i=0; i < CBT_DEFAULT_MIN_CIRCUITS_TO_OBSERVE; i++) {
       build_time_t sample = circuit_build_times_generate_sample(&initial,0,1);
 
-      if (sample > close_ms) {
-        circuit_build_times_add_time(&estimate, CBT_BUILD_ABANDONED);
-      } else {
-        circuit_build_times_add_time(&estimate, sample);
-      }
+      circuit_build_times_add_time(&estimate, sample);
     }
     circuit_build_times_update_alpha(&estimate);
     timeout1 = circuit_build_times_calculate_timeout(&estimate,
@@ -525,119 +612,6 @@ test_circuit_timeout(void *arg)
   testing_disable_deterministic_rng();
 }
 
-/** Test encoding and parsing of rendezvous service descriptors. */
-static void
-test_rend_fns(void *arg)
-{
-  rend_service_descriptor_t *generated = NULL, *parsed = NULL;
-  char service_id[DIGEST_LEN];
-  char service_id_base32[REND_SERVICE_ID_LEN_BASE32+1];
-  const char *next_desc;
-  smartlist_t *descs = smartlist_new();
-  char computed_desc_id[DIGEST_LEN];
-  char parsed_desc_id[DIGEST_LEN];
-  crypto_pk_t *pk1 = NULL, *pk2 = NULL;
-  time_t now;
-  char *intro_points_encrypted = NULL;
-  size_t intro_points_size;
-  size_t encoded_size;
-  int i;
-
-  (void)arg;
-
-  /* Initialize the service cache. */
-  rend_cache_init();
-
-  pk1 = pk_generate(0);
-  pk2 = pk_generate(1);
-  generated = tor_malloc_zero(sizeof(rend_service_descriptor_t));
-  generated->pk = crypto_pk_dup_key(pk1);
-  crypto_pk_get_digest(generated->pk, service_id);
-  base32_encode(service_id_base32, REND_SERVICE_ID_LEN_BASE32+1,
-                service_id, REND_SERVICE_ID_LEN);
-  now = time(NULL);
-  generated->timestamp = now;
-  generated->version = 2;
-  generated->protocols = 42;
-  generated->intro_nodes = smartlist_new();
-
-  for (i = 0; i < 3; i++) {
-    rend_intro_point_t *intro = tor_malloc_zero(sizeof(rend_intro_point_t));
-    crypto_pk_t *okey = pk_generate(2 + i);
-    intro->extend_info = tor_malloc_zero(sizeof(extend_info_t));
-    intro->extend_info->onion_key = okey;
-    crypto_pk_get_digest(intro->extend_info->onion_key,
-                         intro->extend_info->identity_digest);
-    //crypto_rand(info->identity_digest, DIGEST_LEN); /* Would this work? */
-    intro->extend_info->nickname[0] = '$';
-    base16_encode(intro->extend_info->nickname + 1,
-                  sizeof(intro->extend_info->nickname) - 1,
-                  intro->extend_info->identity_digest, DIGEST_LEN);
-    /* Does not cover all IP addresses. */
-    tor_addr_from_ipv4h(&intro->extend_info->addr, crypto_rand_int(65536));
-    intro->extend_info->port = 1 + crypto_rand_int(65535);
-    intro->intro_key = crypto_pk_dup_key(pk2);
-    smartlist_add(generated->intro_nodes, intro);
-  }
-  int rv = rend_encode_v2_descriptors(descs, generated, now, 0,
-                                      REND_NO_AUTH, NULL, NULL);
-  tt_int_op(rv, OP_GT, 0);
-  rv = rend_compute_v2_desc_id(computed_desc_id, service_id_base32, NULL,
-                               now, 0);
-  tt_int_op(rv, OP_EQ, 0);
-  tt_mem_op(((rend_encoded_v2_service_descriptor_t *)
-             smartlist_get(descs, 0))->desc_id, OP_EQ,
-            computed_desc_id, DIGEST_LEN);
-  rv = rend_parse_v2_service_descriptor(&parsed, parsed_desc_id,
-               &intro_points_encrypted, &intro_points_size, &encoded_size,
-               &next_desc,
-          ((rend_encoded_v2_service_descriptor_t *)smartlist_get(descs, 0))
-                                        ->desc_str, 1);
-  tt_int_op(rv, OP_EQ, 0);
-  tt_assert(parsed);
-  tt_mem_op(((rend_encoded_v2_service_descriptor_t *)
-         smartlist_get(descs, 0))->desc_id,OP_EQ, parsed_desc_id, DIGEST_LEN);
-  tt_int_op(rend_parse_introduction_points(parsed, intro_points_encrypted,
-                                         intro_points_size),OP_EQ, 3);
-  tt_assert(!crypto_pk_cmp_keys(generated->pk, parsed->pk));
-  tt_int_op(parsed->timestamp,OP_EQ, now);
-  tt_int_op(parsed->version,OP_EQ, 2);
-  tt_int_op(parsed->protocols,OP_EQ, 42);
-  tt_int_op(smartlist_len(parsed->intro_nodes),OP_EQ, 3);
-  for (i = 0; i < smartlist_len(parsed->intro_nodes); i++) {
-    rend_intro_point_t *par_intro = smartlist_get(parsed->intro_nodes, i),
-      *gen_intro = smartlist_get(generated->intro_nodes, i);
-    extend_info_t *par_info = par_intro->extend_info;
-    extend_info_t *gen_info = gen_intro->extend_info;
-    tt_assert(!crypto_pk_cmp_keys(gen_info->onion_key, par_info->onion_key));
-    tt_mem_op(gen_info->identity_digest,OP_EQ, par_info->identity_digest,
-               DIGEST_LEN);
-    tt_str_op(gen_info->nickname,OP_EQ, par_info->nickname);
-    tt_assert(tor_addr_eq(&gen_info->addr, &par_info->addr));
-    tt_int_op(gen_info->port,OP_EQ, par_info->port);
-  }
-
-  rend_service_descriptor_free(parsed);
-  rend_service_descriptor_free(generated);
-  parsed = generated = NULL;
-
- done:
-  if (descs) {
-    for (i = 0; i < smartlist_len(descs); i++)
-      rend_encoded_v2_service_descriptor_free_(smartlist_get(descs, i));
-    smartlist_free(descs);
-  }
-  if (parsed)
-    rend_service_descriptor_free(parsed);
-  if (generated)
-    rend_service_descriptor_free(generated);
-  if (pk1)
-    crypto_pk_free(pk1);
-  if (pk2)
-    crypto_pk_free(pk2);
-  tor_free(intro_points_encrypted);
-}
-
 #define ENT(name)                                                       \
   { #name, test_ ## name , 0, NULL, NULL }
 #define FORK(name)                                                      \
@@ -650,7 +624,7 @@ static struct testcase_t test_array[] = {
   { "ntor_handshake", test_ntor_handshake, 0, NULL, NULL },
   { "fast_handshake", test_fast_handshake, 0, NULL, NULL },
   FORK(circuit_timeout),
-  FORK(rend_fns),
+  FORK(circuit_timeout_xm_alpha),
 
   END_OF_TESTCASES
 };
@@ -698,8 +672,10 @@ struct testgroup_t testgroups[] = {
   { "crypto/pem/", pem_tests },
   { "crypto/rng/", crypto_rng_tests },
   { "dir/", dir_tests },
+  { "dir/auth/ports/", dirauth_port_tests },
   { "dir/auth/process_descs/", process_descs_tests },
   { "dir/md/", microdesc_tests },
+  { "dirauth/dirvote/", dirvote_tests},
   { "dir/voting/flags/", voting_flags_tests },
   { "dir/voting/schedule/", voting_schedule_tests },
   { "dir_handle_get/", dir_handle_get_tests },
@@ -720,16 +696,18 @@ struct testgroup_t testgroups[] = {
   { "hs_descriptor/", hs_descriptor },
   { "hs_dos/", hs_dos_tests },
   { "hs_intropoint/", hs_intropoint_tests },
+  { "hs_metrics/", hs_metrics_tests },
   { "hs_ntor/", hs_ntor_tests },
+  { "hs_ob/", hs_ob_tests },
   { "hs_service/", hs_service_tests },
-  { "introduce/", introduce_tests },
   { "keypin/", keypin_tests },
-  { "legacy_hs/", hs_tests },
   { "link-handshake/", link_handshake_tests },
   { "mainloop/", mainloop_tests },
+  { "metrics/", metrics_tests },
   { "netinfo/", netinfo_tests },
   { "nodelist/", nodelist_tests },
   { "oom/", oom_tests },
+  { "onion-handshake/ntor-v3/", ntor_v3_tests },
   { "oos/", oos_tests },
   { "options/", options_tests },
   { "options/act/", options_act_tests },
@@ -749,7 +727,6 @@ struct testgroup_t testgroups[] = {
   { "relay/" , relay_tests },
   { "relaycell/", relaycell_tests },
   { "relaycrypt/", relaycrypt_tests },
-  { "rend_cache/", rend_cache_tests },
   { "replaycache/", replaycache_tests },
   { "router/", router_tests },
   { "routerkeys/", routerkeys_tests },
@@ -759,6 +736,7 @@ struct testgroup_t testgroups[] = {
   { "sendme/", sendme_tests },
   { "shared-random/", sr_tests },
   { "socks/", socks_tests },
+  { "statefile/", statefile_tests },
   { "stats/", stats_tests },
   { "status/" , status_tests },
   { "storagedir/", storagedir_tests },
